//********************************************************************
//*** ResourcePool/mysql/Transaction.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: Transaction.cc,v 1.2 2008/01/27 16:42:28 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/mysql/Transaction.hh"

#include <mysql/mysql.h>

namespace fatalmind {
namespace mysql {

Transaction::Transaction(const Command<fatalmind::ResourceType::mysql>& a_cmd) 
: cmd(a_cmd.clone<Command<fatalmind::ResourceType::mysql> >())
{
}


Transaction::Transaction(const ResourcePool<ResourceType>& p, const Command<fatalmind::ResourceType::mysql>& a_cmd)
: cmd(a_cmd.clone<Command<fatalmind::ResourceType::mysql> >())
{
}


Transaction::Transaction() 
: cmd()
{
}


Transaction::Transaction(const Transaction& rho) 
: cmd(rho.cmd->clone<Command<fatalmind::ResourceType::mysql> >())
{
}


Transaction::~Transaction() {
}
void Transaction::execute(mysqlResource& hand) {
	if (cmd.get()) {
		try {
			FinallyTxDepth txCounter(hand);
			startTransaction(hand);
			cmd->execute(hand);
			commit(hand);
		} catch (...) {
			rollback(hand);
			throw;
		}
	}
}

#define MYSQL_START_TRANSACTION "START TRANSACTION"
#define MYSQL_START_TRANSACTION_SZ sizeof(MYSQL_START_TRANSACTION)

void Transaction::startTransaction(mysqlResource& hand) 
{
	MYSQL& dbh = hand.get_plain_resource();

	if (int rc = mysql_real_query(&dbh
		, MYSQL_START_TRANSACTION
		, MYSQL_START_TRANSACTION_SZ)
	) {
		MysqlException::throwit(rc, mysql_error(&dbh), mysql_errno(&dbh), MYSQL_START_TRANSACTION);
	}
}


void Transaction::commit(mysqlResource& hand) {
	MYSQL& dbh = hand.get_plain_resource();

	if (int rc = mysql_commit(&dbh)) {
		MysqlException::throwit(rc, mysql_error(&dbh), mysql_errno(&dbh), "COMMIT");
	}
}


void Transaction::rollback(mysqlResource& hand) {
	MYSQL& dbh = hand.get_plain_resource();

	if (int rc = mysql_rollback(&dbh)) {
		MysqlException::throwit(rc, mysql_error(&dbh), mysql_errno(&dbh), "ROLLBACK");
	}
}

Clone* Transaction::DoClone() const {
	return new Transaction(*this);
}

} // namespace mysql
} // namespace fatalmind
