//********************************************************************
//*** ResourcePool/oracle/StatementCache.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: StatementCache.cc,v 1.12 2008/01/27 16:42:29 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#include "StatementCache.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

namespace fatalmind {
namespace oracle {

StatementCache::StatementCache(unsigned int maxsz, StatementCache* parent) 
: _maxsz(maxsz)
, _parent(parent)
, _map()
{
}

StatementCache::~StatementCache() 
{
}

StatementCache::handle_t& StatementCache::open(
	  const oracleStatement& sql
	, bool parentcache
) {
	if (parentcache && _parent != 0) {
		return _parent->open(sql);
	} else {
		if (_parent != 0) {
			handle_t* parrv = _parent->check(sql);
			if (parrv != 0) {
				return *parrv;
			}
		}
		map_t::iterator elm = _map.find(sql);
		map_t::iterator end = _map.end();
		if (elm != end) {
			// found in cache
			return elm->second;
		} else {
			// not found in cache
			chksize();
			handle_t hand(new OCIStatementHandle(sql));
			std::pair<map_t::iterator, bool> rv = _map.insert(std::make_pair(sql, hand));
			if (rv.second) {
				return (rv.first)->second;
			} else {
				OracleExceptionFactory::throwit("StatementCache.insert", "GPHQCPPLC", -100, "The ResourcePool<ResourceType::oracle> was not able to place a statement handle in the cache. This is an 'impossible' exception, contact development", sql.getSQL());
			}
		}
	}
}

StatementCache::handle_t* StatementCache::check(const oracleStatement& sql) 
{
	map_t::iterator elm = _map.find(sql);
	map_t::iterator end = _map.end();
	if (elm != end) {
		return &elm->second;
	} else {
		return 0;
	}
}

void StatementCache::chksize() 
{
	if (_map.size() >= _maxsz) {
		expire();
	}
}

// TODO: untested yet, make testcases
void StatementCache::expire() 
{
	map_t::iterator end = _map.end();
	map_t::iterator min = end;
	for (map_t::iterator i = _map.begin(); i != end; ++i) {
		if (min == end) {
			min = i;
		} else {
			if (i->second->getEfficienty() < min->second->getEfficienty()) {
				min = i;
			}
		}
	}

	if (min != end) {
		_map.erase(min);
	}
}


}
}
